@@ -27,4 +27,7 @@ void save_save_as(GtkWindow* parent);
 void controls_setup();
 void controls_dialog(GtkWindow* parent);
 
+/* about.c */
+void about_dialog(GtkWindow* parent);
+
 #endif
