@@ -225,6 +225,10 @@ static const ButtonEntry buttons[] = {
 	BUTTON_INITIALIZER(RIGHT, "right"),
 	ACTION_INITIALIZER(Quit, "quit"),
 	ACTION_INITIALIZER(ToggleFullscreen, "fullscreen"),
+	ACTION_INITIALIZER(QuickLoad1, "quickload1"),
+	ACTION_INITIALIZER(QuickSave1, "quicksave1"),
+	ACTION_INITIALIZER(QuickLoad2, "quickload2"),
+	ACTION_INITIALIZER(QuickSave2, "quicksave2"),
 	BUTTON_LAST
 };
 
@@ -234,9 +238,10 @@ static void createActionMappingsOnly()
 	ZeroMemory(Config.joypad1Mapping, sizeof(Config.joypad1Mapping));
 	ZeroMemory(Config.action, sizeof(Config.action));
 	
-	// Map quit to fullscreen and escape
+	// Map quit to fullscreen, escape and task switch.
 	Config.action[72] = kActionQuit;
 	Config.action[9] = kActionQuit;
+	Config.action[71] = kActionQuit;
 }
 
 static void parseGConfKeyMappings()
@@ -287,5 +292,10 @@ static void parseGConfKeyMappings()
 			Config.action[72] = kActionQuit;
 		}
 	}
+
+	// If task switch key is not mapped, map it to Quit by default.
+	if (!Config.action[71] && !Config.joypad1Mapping[71]) {
+		Config.action[71] = kActionQuit;
+	}
 }
 
